# -*- coding: utf-8 -*-
"""
Created on Tue Jan  1 21:04:25 2019

@author: yoelr
"""

from .color import Color
from .palette import Palette
from .color_wheel import ColorWheel
from . import utils
from .utils import *

__version__ = '0.3.2'

__all__ = ('Color', 'Palette', 'ColorWheel', *utils.__all__)


# -*- coding: utf-8 -*-
"""
Created on Tue Jan  1 21:02:22 2019

@author: yoelr
"""
from .palette import Palette
from math import floor, ceil
from .color import Color
from .utils import view

class ColorWheel:
    """
    Create a ColorWheel object that cycles through Color objects.
    
    Parameters
    ----------
        colors: tuple[Color]
            Colors to cycle through.
    
    Examples
    --------
    :doc:`GettingStarted`
    
    """
    __slots__ = ('colors', 'index')
    
    def __init__(self, colors):
        self.colors = tuple(colors)
        self.restart()

    def view(self, *args, **kwargs):
        return view(self.colors, *args, **kwargs)

    def interpolate(self, x):
        colors = self.colors
        y = x * (len(colors) - 1)
        lb = int(floor(y))
        ub = int(ceil(y))
        s = y - lb
        return Color(
            fg = (1 - s) * colors[lb].RGB + s * colors[ub].RGB
        )
    
    def restart(self):
        self.index = 0
    
    def __getitem__(self, index):
        return self.colors[index % len(self.colors)]
    
    def next(self):
        colors = self.colors
        index = self.index
        color = colors[index]
        self.index = (index + 1) % len(colors)
        return color
    
    __repr__ = Palette.__repr__
    _ipython_display_ = Palette._ipython_display_


# -*- coding: utf-8 -*-
"""
Created on Tue Jan  1 21:02:22 2019

@author: yoelr
"""
from .color import Color
import colorpalette

class Palette:
    """
    Create a Palette object that manages Color objects.
    
    Parameters
    ----------
        **colors: Color
            Key/Color pairs to initialize attributes.
    
    Examples
    --------
    :doc:`GettingStarted`
    
    """
    
    def __init__(self, **colors):
        for key, color in colors.items():
            if key != color.ID: color = color.copy(key)
            setattr(self, key, color)
    
    def view(self, *args, **kwargs):
        return colorpalette.view(tuple(self), *args, **kwargs)
    
    def wheel(self):
        return colorpalette.ColorWheel(self)
    
    def __setattr__(self, attr, new_color):
        if isinstance(new_color, Color):
            super().__setattr__(attr, new_color)
        else:
            raise TypeError(f'Attribute must be an instance of Color, not {type(new_color).__name__}')
    
    def __len__(self):
        return len(self.__dict__)
    
    def __iter__(self):
        return (color for color in self.__dict__.values())
    
    def _ipython_display_(self):
        print(repr(self))
    
    def __repr__(self):
        return f"{type(self).__name__}({', '.join([color(name) for name, color in self.__dict__.items()])})"



    
    


# -*- coding: utf-8 -*-
"""
Created on Tue Jan  1 20:56:58 2019

@author: yoelr
"""
from colorama import Style
from .utils import rgb2hex, decode_ansi, rgb_tint, rgb_shade
import numpy as np
import colors


class Color:
    """
    Create a Color object that can color/style strings. Color objects are an 
    extension of the color function in the `ansicolors` 
    package <https://pypi.org/project/ansicolors/>`__.
    
    Parameters
    ----------
    ID: str
        Name of color.
    fg: str
        Foreground color.
    bg: str
        Background color.
    style: str
        Options include 'bold', 'underline', or None
    
    Notes
    -----
    Both *fg* and *bg* should be one of the following [-]:
        * 3-element tuple or list of int, each valued 0 to 255 (e.g. (255, 218, 185)),
        * string containing a CSS-compatible color name (e.g. 'peachpuff'),
        * string containing a CSS-style hex value (e.g. '#aaa' or '#8a2be2')
        * string containing a CSS-style RGB notation (e.g. 'rgb(102,51,153)')
        * None
    
    Examples
    --------
    :doc:`GettingStarted`
    
    """
    __slots__ = ('ID', '_ansi', '_RGB', '_RGB_bg', '_style')
    
    cached = {}
    
    def __new__(self, ID='', fg=None, bg=None, style=None):
        if ID and fg is None and bg is None and style is None:
            return self.cached[ID]
        if isinstance(fg, np.ndarray): fg = tuple([int(i) for i in fg])
        if isinstance(bg, np.ndarray): bg = tuple([int(i) for i in bg])
        ansi = colors.color('', fg, bg, style).replace(Style.RESET_ALL, '')
        return self.from_ansi(ID, ansi)
    
    def copy(self, ID=None):
        copy = super().__new__(self.__class__)
        copy.ID = self.ID if ID is None else ID
        copy._ansi = self._ansi
        return copy
        
    @property
    def ansi(self):
        """[str] Ansi color code."""
        return self._ansi
    @ansi.setter
    def ansi(self, ansi):
        self._ansi = ansi
        if hasattr(self, '_RGB'): del self._RGB
        if hasattr(self, '_RGB_bg'): del self._RGB_bg
        if hasattr(self, '_style'): del self._style
    
    @classmethod
    def from_ansi(cls, ID='', ansi=''):
        """Create a Color object with ansi color code string."""
        self = object.__new__(cls)
        self._ansi = ansi
        if ID and ID not in cls.cached: cls.cached[ID] = self
        self.ID = ID
        return self
    
    def __call__(self, string):
        return self + str(string) + Style.RESET_ALL 

    def __add__(self, string):
        return self._ansi + string
    
    def __radd__(self, string):
        return string + self._ansi

    def invert(self, ID=None):
        """Return a new Color object with their foreground and background swapped."""
        return type(self)(ID=ID or f"{self.ID}-inverted", 
                          fg=self.HEX_bg, bg=self.HEX, style=self._style)

    @property
    def RGB(self):
        """[array] Foreground color in RGB on a 0-255 scale."""
        if not hasattr(self, '_RGB'):
            self._RGB, self._RGB_bg, self._style = decode_ansi(self._ansi)
        return self._RGB
    @property
    def RGB_bg(self):
        """[array] Background color in RGB on a 0-255 scale."""
        if not hasattr(self, '_RGB_bg'):
            self._RGB, self._RGB_bg, self._style = decode_ansi(self._ansi)
        return self._RGB_bg
    @property
    def style(self):
        """[str] Font style."""
        if not hasattr(self, '_style'):
            self._RGB, self._RGB_bg, self._style = decode_ansi(self._ansi)
        return self._style
    
    @property
    def RGBn(self):
        """[array] Foreground color in RGB on a normalized 0-1 scale."""
        RGB = self.RGB
        return None if RGB is None else self.RGB/255.
    @property
    def RGBn_bg(self):
        """[array] Background color in RGB on a normalized 0-1 scale."""
        RGB_bg = self.RGB_bg
        return None if RGB_bg is None else self.RGB_bg/255.
    
    @property
    def HEX(self):
        """[str] Foreground color in HEX code."""
        RGB = self.RGB
        return None if RGB is None else rgb2hex(RGB)
    @property
    def HEX_bg(self):
        """[str] Background color in HEX code."""
        RGB_bg = self.RGB_bg
        return None if RGB_bg is None else rgb2hex(RGB_bg)

    def tint(self, percent, ID=None):
        """Return a new Color object tinted by given percent."""
        fg, bg, style = decode_ansi(self._ansi)
        if fg is not None:
            fg = rgb_tint(fg, percent)
        if bg is not None:
            bg = rgb_tint(bg, percent)
        if not ID:
            *previous_tint, ID = self.ID.split(' tinted ')
            if len(previous_tint) == 1:
                try:
                    p, *_ = previous_tint[0].split('%')
                    p = float(p)
                    percent = p + (100.-p)*percent/100.
                except: pass
            ID = f'{percent}% tinted {ID}'
        return Color(ID, fg, bg, style)

    def shade(self, percent, ID=None):
        """Return a new Color object shaded by given percent."""
        fg, bg, style = decode_ansi(self._ansi)
        if fg is not None:
            fg = rgb_shade(fg, percent)
        if bg is not None:
            bg = rgb_shade(bg, percent)
        if not ID:
            *previous_shade, ID = self.ID.split(' shaded ')
            if len(previous_shade):
                try:
                    p, *_ = previous_shade[0].split('%')
                    p = float(p)
                    percent = p + (100.-p)*percent/100.
                except: pass
            ID = f'{percent}% shaded {ID}'
        return Color(ID, fg, bg, style)
    
    def show(self):
        print(repr(self))
    _ipython_display_ = show  
    
    def __str__(self):
        return self._ansi
    
    def __repr__(self):
        if self.ID:
            name = self.ID
        else:
            fg = self.RGB
            bg = self.RGB_bg
            style = self.style
            if isinstance(fg, np.ndarray): fg = tuple(fg)
            if isinstance(bg, np.ndarray): bg = tuple(bg)
            name = f"{type(self).__name__}(fg={fg}, bg={bg}, style={repr(style)})"
        return f'{self}{name}{Style.RESET_ALL}'
    
Color.cached = dict((i, Color(i, fg=i)) for i in colors.css_colors.keys())
Color.reset = Color.from_ansi('reset', Style.RESET_ALL)



# -*- coding: utf-8 -*-
"""
Created on Sun Dec 30 18:42:27 2018

@author: yoelr
"""

from setuptools import setup

setup(
      name = 'colorpalette',
      packages = ['colorpalette'],
      license='MIT',
      version = '0.3.2',
      description = 'Objects for coloring strings',
      long_description=open('README.rst').read(),
      author = 'Yoel Cortes-Pena',
      install_requires=['ansicolors', 'colorama', 'numpy'],
      package_data = {'colorpalette': []},
      platforms=["Windows"],
      author_email = 'yoelcortes@gmail.com',
      url = 'https://github.com/yoelcortes/colorpalette',
      download_url = 'https://github.com/yoelcortes/colorpalette.git'
      )
      


# -*- coding: utf-8 -*-
"""
Created on Thu Aug 15 03:17:08 2019

@author: yoelr
"""
import numpy as np

RGBs = (np.array([0  , 0  , 0  ]), # black
        np.array([170, 0  , 0  ]), # red
        np.array([0  , 170, 0  ]), # green
        np.array([170, 170, 0  ]), # yellow
        np.array([0  , 0  , 170]), # blue
        np.array([170, 0  , 170]), # magenta
        np.array([0  , 170, 170]), # cyan
        np.array([255, 255, 255])) # white

__all__ = ('decode_ansi', 'rgb2hex', 'rgb_tint', 'rgb_shade', 'view')


def view(colors, names=None, format='HEX'):
    import matplotlib.pyplot as plt
    N = np.ceil(len(colors) / 6)
    ymax = N
    xmax = 6
    plt.axis([0, xmax, 0, N])
    plt.axis('off')
    position = 0
    ax = plt.gca()
    for i in colors:
        x = 0.5 + position % xmax
        y = ymax - np.floor(position / xmax) - 0.5
        position += 1
        ax.text(x, y + 0.3, i.ID, fontsize=8, horizontalalignment='center', 
                verticalalignment='center')
        ax.text(x, y + 0.5, getattr(i, format), fontsize=8, horizontalalignment='center', 
                verticalalignment='center')
        assert x < xmax
        assert y + 0.3 < ymax
        circle = plt.Circle([x, y], radius=0.2, color=i.RGBn)
        ax.add_artist(circle)

def rgb2hex(rgb):
    R, G, B = rgb
    return f"#{int(R):02x}{int(G):02x}{int(B):02x}"

def rgb_tint(rgb, percent):
    """Return tinted rgb by given percent."""
    white = np.array([255, 255, 255])
    vector = white-rgb
    return rgb + vector * percent/100

def rgb_shade(rbg, percent):
    """Return shaded rgb by given percent."""
    return rbg * (100-percent)/100

def decode_ansi(ansi):
    """Return fg, bg, and style from ansi string."""
    ansi = ansi.lstrip('\x1b[')
    first, *nums = ansi[:ansi.index('m')].split(';')
    fgbgstyle = []
    for i in '34':
        if first is False:
            rgb = None
        elif first[0] == i:
            n = first[1]
            if n == '8':
                nums = nums[1:]
                rgb = np.zeros(3)
                for i, n in zip(range(3), nums): rgb[i] = n
                nums = nums[i+1:]
            elif n == '9':
                rgb = None
            else:
                rgb = RGBs[int(n)]
        else:
            rgb = None
        fgbgstyle.append(rgb)
        if nums: first, *nums = nums
        else: first = False
    n = int(first)
    if n == 0:
        fgbgstyle.append(None)
    elif n == 1:
        fgbgstyle.append('bold')
    elif n == 2:
        fgbgstyle.append('faint')
    elif n == 3:
        fgbgstyle.append('italic')
    elif n == 4:
        fgbgstyle.append('underline')
    else:
        fgbgstyle.append(None)
    return fgbgstyle